import { CommonModule } from '@angular/common';
import { HttpClient, HttpParams, HttpRequest, HttpClientModule } from '@angular/common/http';
import { InjectionToken, Injectable, Inject, NgModule, defineInjectable, inject } from '@angular/core';
import { forOwn, isArray, mapValues, isBoolean, cloneDeep, omit } from 'lodash';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const EnvironmentConfigService = new InjectionToken('EnvironmentConfigService');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const HttpMethods = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    PATCH: 'PUT',
    DELETE: 'DELETE',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const methodsWithBody = [HttpMethods.POST, HttpMethods.PUT, HttpMethods.PATCH];
/** @type {?} */
const uploadingSection = ['media'];
class HttpService {
    /**
     * @param {?} http
     * @param {?} config
     */
    constructor(http, config) {
        this.http = http;
        this.config = config;
    }
    /**
     * @private
     * @param {?} urlParams
     * @param {?} method
     * @param {?} options
     * @return {?}
     */
    handle(urlParams, method, options) {
        options = this.filterParams(urlParams, options);
        if (methodsWithBody.indexOf(method) === -1) {
            options = {
                params: new HttpParams({
                    fromObject: options
                })
            };
        }
        /** @type {?} */
        let request = new HttpRequest(method, this.buildUrl(urlParams), options, {
            responseType: urlParams.responseType || 'json'
        });
        // add header to prevent token checking for public requests
        if (urlParams.insecure === true) {
            request = request.clone({ setHeaders: { Insecure: '1' } });
            delete urlParams.insecure;
        }
        return this.http.request(request)
            .toPromise()
            .then((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            return {
                result: true,
                data: (urlParams.rawResponse === true) ? event : event.body
            };
        }))
            .catch((/**
         * @param {?} error
         * @return {?}
         */
        (error) => {
            return {
                result: false,
                error: (urlParams.rawResponse === true) ? error : error.error
            };
        }));
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    get(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.GET, params);
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    search(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.GET, params);
    }
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    save(urlParams, data) {
        return this.handle(urlParams, HttpMethods.POST, data);
    }
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    update(urlParams, data) {
        return this.handle(urlParams, HttpMethods.PUT, data);
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    delete(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.DELETE, params);
    }
    /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @private
     * @param {?} urlParams
     * @param {?} options
     * @return {?}
     */
    filterParams(urlParams, options) {
        /** @type {?} */
        let params = {};
        forOwn(options, (/**
         * @param {?} value
         * @param {?} key
         * @return {?}
         */
        (value, key) => {
            if (isArray(value) && key.indexOf('[]') === -1) {
                key += '[]';
            }
            if (value !== null && value !== undefined) {
                params[key] = value;
            }
        }));
        // convert boolean values to numeric except media content
        if (uploadingSection.indexOf(urlParams.section) === -1) {
            params = mapValues(params, (/**
             * @param {?} value
             * @return {?}
             */
            value => isBoolean(value) ? +value : value));
        }
        return params;
    }
    /**
     * Make url like `/section/id/cmd/secondId`
     * @private
     * @param {?} urlParams
     * @return {?}
     */
    buildUrl(urlParams) {
        /** @type {?} */
        const url = [`${this.config.url}/${urlParams.section}`];
        if (urlParams.id !== undefined) {
            url.push(`${urlParams.id}`);
        }
        if (urlParams.cmd) {
            url.push(`${urlParams.cmd}`);
        }
        if (urlParams.secondId) {
            url.push(`${urlParams.secondId}`);
        }
        return url.join('/');
    }
}
HttpService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
HttpService.ctorParameters = () => [
    { type: HttpClient },
    { type: undefined, decorators: [{ type: Inject, args: [EnvironmentConfigService,] }] }
];
/** @nocollapse */ HttpService.ngInjectableDef = defineInjectable({ factory: function HttpService_Factory() { return new HttpService(inject(HttpClient), inject(EnvironmentConfigService)); }, token: HttpService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const RequestTypes = {
    GET: 0,
    SEARCH: 1,
    ADD: 2,
    UPDATE: 3,
    DELETE: 4,
    RESTORE: 5,
};
RequestTypes[RequestTypes.GET] = 'GET';
RequestTypes[RequestTypes.SEARCH] = 'SEARCH';
RequestTypes[RequestTypes.ADD] = 'ADD';
RequestTypes[RequestTypes.UPDATE] = 'UPDATE';
RequestTypes[RequestTypes.DELETE] = 'DELETE';
RequestTypes[RequestTypes.RESTORE] = 'RESTORE';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
class GetRequest {
    /**
     * @param {?} service
     * @param {?} queryParams
     */
    constructor(service, queryParams) {
        this.service = service;
        this.queryParams = queryParams;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    url(urlParams) {
        (/** @type {?} */ (this)).urlParams = urlParams;
        return (/** @type {?} */ (this));
    }
    /**
     * Implementation of the adding `with_trashed` into request
     *
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    withTrashed(value = true) {
        (/** @type {?} */ (this)).queryParams.with_trashed = value;
        return (/** @type {?} */ (this));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiGetRequest extends GetRequest {
    /**
     * @param {?} service
     * @param {?} queryParams
     */
    constructor(service, queryParams) {
        super(service, queryParams);
        this.service = service;
        this.queryParams = queryParams;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} id
     * @return {THIS}
     */
    id(id) {
        (/** @type {?} */ (this)).queryParams.id = id;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    with(value) {
        (/** @type {?} */ (this)).queryParams.with = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @return {?}
     */
    execute() {
        /** @type {?} */
        const params = cloneDeep(this.queryParams);
        delete params.id;
        return this.service.get(this.queryParams.id, params || {});
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiSearchRequest extends GetRequest {
    /**
     * @param {?} service
     * @param {?} queryParams
     */
    constructor(service, queryParams) {
        super(service, queryParams);
        this.service = service;
        this.queryParams = queryParams;
    }
    // TODO: try to use decorator to change url (as parameter of function) + use declare in service
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    query(value) {
        (/** @type {?} */ (this)).queryParams.query = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    page(value) {
        (/** @type {?} */ (this)).queryParams.page = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    perPage(value) {
        (/** @type {?} */ (this)).queryParams.per_page = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?} value
     * @return {THIS}
     */
    filterBy(field, value) {
        (/** @type {?} */ (this)).queryParams[field] = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?=} isDescending
     * @return {THIS}
     */
    orderBy(field, isDescending = false) {
        (/** @type {?} */ (this)).queryParams.order_by = field;
        (/** @type {?} */ (this)).queryParams.desc = isDescending;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    all(value = true) {
        (/** @type {?} */ (this)).queryParams.all = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    with(value) {
        (/** @type {?} */ (this)).queryParams.with = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @return {?}
     */
    execute() {
        return this.service.search(this.queryParams, this.urlParams);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
class PostRequest {
    /**
     * @param {?} service
     * @param {?} entity
     */
    constructor(service, entity) {
        this.service = service;
        this.entity = entity;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    url(urlParams) {
        (/** @type {?} */ (this)).urlParams = urlParams;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} data
     * @return {THIS}
     */
    data(data) {
        (/** @type {?} */ (this)).entity = data;
        return (/** @type {?} */ (this));
    }
    // filter(filter: ((data: any) => T) | string[]): ApiBodyRequest<T> {
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} filter
     * @return {THIS}
     */
    filter(filter) {
        if (typeof filter === 'function') {
            (/** @type {?} */ (this)).entity = filter((/** @type {?} */ (this)).entity);
        }
        else {
            (/** @type {?} */ (this)).entity = (/** @type {?} */ (omit((/** @type {?} */ (this)).entity, filter)));
        }
        return (/** @type {?} */ (this));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiAddRequest extends PostRequest {
    /**
     * @return {?}
     */
    execute() {
        return this.service.add(this.entity, this.urlParams);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiUpdateRequest extends PostRequest {
    /**
     * @return {?}
     */
    execute() {
        return this.service.update(this.entity, this.urlParams);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiService {
    /**
     * @param {?} httpService
     */
    constructor(httpService) {
        this.httpService = httpService;
        this.section = '';
        this.omittedFields = [];
        this.isInsecure = false;
    }
    /**
       *
       * @param {number | string} id
       * @param {object} params
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // get<Y>(id: number | string, params?: Omit<GetParams, 'id'>, urlParams?: UrlParams): Promise<ApiResponse<Y>>;
    /**
     *
     * @param {?} id
     * @param {?=} params
     * @param {?=} urlParams
     * @return {?}
     */
    get(id, params, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: id,
            insecure: this.isInsecure
        };
        return this.httpService.get(url, params);
    }
    /**
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // add<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<T>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    add(data, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService.save(url, this.omitFields(data));
    }
    /**
       *
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<void>>}
       */
    // update<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    update(data, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: data.id,
            insecure: this.isInsecure
        };
        /** @type {?} */
        const dataWithoutId = cloneDeep(data);
        delete dataWithoutId.id;
        return this.httpService.update(url, dataWithoutId);
    }
    /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    delete(id, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: id,
            insecure: this.isInsecure
        };
        return this.httpService.delete(url);
    }
    /**
     * Soft delete list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    deleteGroup(ids, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService
            .delete(url, {
            'ids[]': ids
        });
    }
    /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    restore(id, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: id,
            cmd: 'restore',
            insecure: this.isInsecure
        };
        return this.httpService.save(url, {});
    }
    /**
     * Restore list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    restoreGroup(ids, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: 'restore',
            insecure: this.isInsecure
        };
        return this.httpService
            .save(url, {
            'ids[]': ids
        });
    }
    /**
       *
       * @param {SearchParams} searchParams
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y[]>>}
       */
    // search<Y extends ApiEntity>(searchParams: SearchParams, urlParams?: UrlParams): Promise<ApiResponse<ApiPagination<Y[]>>>;
    /**
     *
     * @param {?} searchParams
     * @param {?=} urlParams
     * @return {?}
     */
    search(searchParams, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService
            .search(url, searchParams)
            .then((/**
         * @param {?} response
         * @return {?}
         */
        response => {
            if (this.httpService.config.adaptivePagination && searchParams && searchParams.all) {
                // FIXME: temporary solution until backend will be updated to the new version
                response.data = {
                    current_page: 1,
                    data: response.data,
                    first_page_url: '',
                    from: 1,
                    last_page: 1,
                    last_page_url: '',
                    next_page_url: '',
                    path: '',
                    per_page: response.data.length,
                    prev_page_url: '',
                    to: response.data.length,
                    total: response.data.length
                };
            }
            return response;
        }));
    }
    /**
     * @param {?} method
     * @param {?=} params
     * @return {?}
     */
    request(method, params = {}) {
        /** @type {?} */
        let request;
        switch (method) {
            case RequestTypes.SEARCH: {
                request = new ApiSearchRequest(this, params);
                break;
            }
            case RequestTypes.GET: {
                request = new ApiGetRequest(this, params);
                break;
            }
            case RequestTypes.ADD: {
                request = new ApiAddRequest(this, params);
                break;
            }
            case RequestTypes.UPDATE: {
                request = new ApiUpdateRequest(this, params);
                break;
            }
        }
        return request;
    }
    /**
     * @protected
     * @template Y
     * @param {?} data
     * @return {?}
     */
    omitFields(data) {
        return omit(data, ...this.omittedFields);
    }
}
ApiService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
ApiService.ctorParameters = () => [
    { type: HttpService }
];
/** @nocollapse */ ApiService.ngInjectableDef = defineInjectable({ factory: function ApiService_Factory() { return new ApiService(inject(HttpService)); }, token: ApiService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// export * from './lib/services/auth.service';
// export * from './lib/interceptors/api-auth.interceptor';
// export * from './lib/interceptors/api-jwt.interceptor';
class ApiLibModule {
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: ApiLibModule,
            providers: [
                { provide: EnvironmentConfigService, useValue: config },
                HttpService,
                ApiService
                // { provide: HttpService, useClass: 'ApiHttpService' },
                // { provide: AuthService, useClass: 'ApiAuthService' },
            ]
        };
    }
}
ApiLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    CommonModule,
                    HttpClientModule
                ],
                exports: [],
                providers: [
                    HttpService,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiLibModule, HttpMethods, RequestTypes, ApiAddRequest, ApiGetRequest, ApiSearchRequest, ApiUpdateRequest, GetRequest, PostRequest, EnvironmentConfigService, HttpService, ApiService };

//# sourceMappingURL=api-lib.js.map