import { HttpService } from './http.service';
import { RequestTypes } from '../enums/request-types.enum';
import { ApiEntity } from '../interfaces/api-entity';
import { GetParams } from '../interfaces/get-params';
import { UrlParams } from '../interfaces/url-params';
import { ApiResponse } from '../interfaces/api-response';
import { SearchParams } from '../interfaces/search-params';
import { ApiPagination } from '../interfaces/api-pagination';
import { ApiGetRequest } from '../requests/api-get.request';
import { ApiSearchRequest } from '../requests/api-search.request';
import { ApiAddRequest } from '../requests/api-add.request';
import { ApiUpdateRequest } from '../requests/api-update.request';
declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
export declare class ApiService<T extends ApiEntity> {
    protected httpService: HttpService;
    protected section: string;
    protected omittedFields: any[];
    protected isInsecure: boolean;
    constructor(httpService: HttpService);
    /**
     *
     * @param {number | string} id
     * @param {object} params
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<Y>>}
     */
    /**
     *
     * @param {number | string} id
     * @param {object} params
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<T>>}
     */
    get(id: number | string, params?: Omit<GetParams, 'id'>, urlParams?: UrlParams): Promise<ApiResponse<T>>;
    /**
     * @param {Y} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<Y>>}
     */
    /**
     *
     * @param {T} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<T>>}
     */
    add(data: Partial<T>, urlParams?: UrlParams): Promise<ApiResponse<T>>;
    /**
     *
     * @param {Y} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    /**
     *
     * @param {ApiEntity} data
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    update(data: Partial<T>, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {number} id
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    delete(id: number, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     * Soft delete list of objects
     * (if backend supports this method)
     * @param {number[]} ids
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    deleteGroup(ids: number[], urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {number} id
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    restore(id: number, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     * Restore list of objects
     * (if backend supports this method)
     * @param {number[]} ids
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<void>>}
     */
    restoreGroup(ids: number[], urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {SearchParams} searchParams
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<Y[]>>}
     */
    /**
     *
     * @param {SearchParams} searchParams
     * @param {UrlParams} urlParams
     * @return {Promise<ApiResponse<T[]>>}
     */
    search(searchParams: SearchParams, urlParams?: UrlParams): Promise<ApiResponse<ApiPagination<T[]>>>;
    request(method: RequestTypes.GET, params?: GetParams): ApiGetRequest<T>;
    request(method: RequestTypes.SEARCH, params?: SearchParams): ApiSearchRequest<T>;
    request(method: RequestTypes.ADD, params?: Partial<ApiEntity>): ApiAddRequest<T>;
    request(method: RequestTypes.UPDATE, params?: Partial<ApiEntity>): ApiUpdateRequest<T>;
    protected omitFields<Y extends Partial<ApiEntity>>(data: T | Y): any;
}
export {};
